@@ -116,6 +116,24 @@ func TestDoHIPsOfBase(t *testing.T) {
 				"2a07:a8c1::c3:a884",
 			),
 		},
+		{
+			base: "https://dns.controld.com/hyq3ipr2ct",
+			want: ips(
+				"76.76.2.22",
+				"76.76.10.22",
+				"2a07:a8c0:0:6:7b5b:5949:35ad:0",
+				"2a07:a8c1:0:6:7b5b:5949:35ad:0",
+			),
+		},
+		{
+			base: "https://dns.controld.com/112233445566778899aabbcc",
+			want: ips(
+				"76.76.2.22",
+				"76.76.10.22",
+				"2a07:a8c0:0:ffff:ffff:ffff:ffff:0",
+				"2a07:a8c1:0:ffff:ffff:ffff:ffff:0",
+			),
+		},
 	}
 	for _, tt := range tests {
 		got := DoHIPsOfBase(tt.base)
