@@ -189,7 +189,8 @@ type BucketedStatsOptions struct {
 	Started *metrics.LabelMap
 
 	// If non-nil, Finished maintains a counter of all requests which
-	// have finished processing (that is, the HTTP handler has returned).
+	// have finished processing with success (that is, the HTTP handler has
+	// returned).
 	Finished *metrics.LabelMap
 }
 
@@ -311,15 +312,50 @@ func (h retHandler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 	}
 
 	var bucket string
+	bumpStartIfNeeded := func() {}
 	if bs := h.opts.BucketedStats; bs != nil {
 		bucket = bs.bucketForRequest(r)
 		if bs.Started != nil {
-			bs.Started.Add(bucket, 1)
+			switch v := bs.Started.Map.Get(bucket).(type) {
+			case *expvar.Int:
+				// If we've already seen this bucket for, count it immediately.
+				v.Add(1)
+			case nil:
+				// Otherwise, for newly seen paths, only count retroactively
+				// (so started-finished doesn't go negative) so we don't fill
+				// this LabelMap up with internet scanning spam.
+				bumpStartIfNeeded = func() { bs.Started.Add(bucket, 1) }
+			}
 		}
 	}
 
 	lw := &loggingResponseWriter{ResponseWriter: w, logf: h.opts.Logf}
-	err := h.rh.ServeHTTPReturn(lw, r)
+
+	// In case the handler panics, we want to recover and continue logging the
+	// error before raising the panic again for the server to handle.
+	var (
+		didPanic bool
+		panicRes any
+	)
+	defer func() {
+		if didPanic {
+			panic(panicRes)
+		}
+	}()
+	runWithPanicProtection := func() (err error) {
+		defer func() {
+			if r := recover(); r != nil {
+				didPanic = true
+				panicRes = r
+				// Even if r is an error, do not wrap it as an error here as
+				// that would allow things like panic(vizerror.New("foo")) which
+				// is really hard to define the behavior of.
+				err = fmt.Errorf("panic: %v", r)
+			}
+		}()
+		return h.rh.ServeHTTPReturn(lw, r)
+	}
+	err := runWithPanicProtection()
 
 	var hErr HTTPError
 	var hErrOK bool
@@ -401,7 +437,15 @@ func (h retHandler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 	}
 
 	if bs := h.opts.BucketedStats; bs != nil && bs.Finished != nil {
-		bs.Finished.Add(bucket, 1)
+		// Only increment metrics for buckets that result in good HTTP statuses.
+		// Otherwise they get full of internet scanning noise. Only filtering 404
+		// gets most of the way there but there are also plenty of URLs that are
+		// almost right but result in 400s too. Seem easier to just only ignore
+		// all 4xx and 5xx.
+		if msg.Code < 400 {
+			bumpStartIfNeeded()
+			bs.Finished.Add(bucket, 1)
+		}
 	}
 
 	if !h.opts.QuietLoggingIfSuccessful || (msg.Code != http.StatusOK && msg.Code != http.StatusNotModified) {
