@@ -6,6 +6,7 @@ package syspolicy
 import (
 	"errors"
 	"sync/atomic"
+	"testing"
 )
 
 var (
@@ -56,3 +57,10 @@ func RegisterHandler(h Handler) {
 		panic("handler was already used before registration")
 	}
 }
+
+func SetHandlerForTest(tb testing.TB, h Handler) {
+	tb.Helper()
+	oldHandler := handler
+	handler = h
+	tb.Cleanup(func() { handler = oldHandler })
+}
